package com.mortals.xhx.modules.implementlist.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.mortals.xhx.common.utils.SnowFlakeUtil;
import com.mortals.xhx.modules.implementlist.dao.RybMatterDao;
import com.mortals.xhx.modules.implementlist.model.RybMatterListEntry;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

/**
 * @author liaozimu
 * @Date: 2021-11-2021/11/2-17:23
 * @Description:
 */
@Slf4j
public class RybMatterImportListener extends AnalysisEventListener<RybMatterListEntry> {

    private RybMatterDao rybMatterDao;

    /**
     * 每隔5条存储数据库，实际使用中可以3000条，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 3000;

    /**
     * 这个集合用于接收 读取Excel文件得到的数据
     */
    List<RybMatterListEntry> entityList = new ArrayList<RybMatterListEntry>();


    public RybMatterImportListener(RybMatterDao rybMatterDao) {
        this.rybMatterDao = rybMatterDao;
    }

    @SneakyThrows
    @Override
    public void invoke(RybMatterListEntry rybMatterListEntry, AnalysisContext analysisContext) {
        rybMatterListEntry.setId(SnowFlakeUtil.get().nextId());
        entityList.add(rybMatterListEntry);
        //达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
        if (entityList.size() >= BATCH_COUNT) {
            rybMatterDao.inserts(entityList);
            //存储完成清理list
            entityList.clear();
        }
        log.info("=======Excel文件<<>>导入成功===========");
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        rybMatterDao.inserts(entityList);
        log.info("=======Excel文件<<>>导入成功===========");
    }
}
